from typing import Annotated, Literal

from pydantic import Field

from docling_jobkit.datamodel.google_drive_coords import GoogleDriveCoordinates
from docling_jobkit.datamodel.http_inputs import FileSource, HttpSource
from docling_jobkit.datamodel.s3_coords import S3Coordinates


class TaskFileSource(FileSource):
    kind: Literal["file"] = "file"


class TaskHttpSource(HttpSource):
    kind: Literal["http"] = "http"


class TaskS3Source(S3Coordinates):
    kind: Literal["s3"] = "s3"


class TaskGoogleDriveSource(GoogleDriveCoordinates):
    kind: Literal["google_drive"] = "google_drive"


TaskSource = Annotated[
    TaskFileSource | TaskHttpSource | TaskS3Source | TaskGoogleDriveSource,
    Field(discriminator="kind"),
]



from typing import Iterator

from docling.datamodel.base_models import DocumentStream

from docling_jobkit.connectors.source_processor import BaseSourceProcessor
from docling_jobkit.datamodel.http_inputs import FileSource, HttpSource


class HttpSourceProcessor(BaseSourceProcessor):
    def __init__(self, source: HttpSource | FileSource):
        super().__init__()
        self._source = source

    def _initialize(self):
        pass

    def _finalize(self):
        pass

    def _fetch_documents(self) -> Iterator[DocumentStream]:
        if isinstance(self._source, FileSource):
            yield self._source.to_document_stream()
        elif isinstance(self._source, HttpSource):
            # TODO: fetch, e.g. using the helpers in docling-core
            raise NotImplementedError()



from io import BytesIO
from typing import Iterator

from docling.datamodel.base_models import DocumentStream

from docling_jobkit.connectors.s3_helper import get_s3_connection, get_source_files
from docling_jobkit.connectors.source_processor import BaseSourceProcessor
from docling_jobkit.datamodel.s3_coords import S3Coordinates


class S3SourceProcessor(BaseSourceProcessor):
    def __init__(self, coords: S3Coordinates):
        super().__init__()
        self._coords = coords

    def _initialize(self):
        self._client, self._resource = get_s3_connection(self._coords)

    def _finalize(self):
        self._client.close()

    def _fetch_documents(self) -> Iterator[DocumentStream]:
        # get list of object_keys
        object_keys = get_source_files(
            s3_source_client=self._client,
            s3_source_resource=self._resource,
            s3_coords=self._coords,
        )

        # download and yield one document at the time
        for obj_key in object_keys:
            # todo. stream is BytesIO
            buffer = BytesIO()
            self._client.download_fileobj(
                Bucket=self._coords.bucket,
                Key=obj_key,
                Fileobj=buffer,
            )
            buffer.seek(0)
            yield DocumentStream(
                name=obj_key,
                stream=buffer,
            )



from abc import ABC, abstractmethod
from contextlib import AbstractContextManager
from typing import Iterator

from docling.datamodel.base_models import DocumentStream


class BaseSourceProcessor(AbstractContextManager, ABC):
    """
    Base class for source processors.
    Handles initialization state and context management.
    """

    def __init__(self):
        self._initialized = False  # Track whether the processor is ready

    def __enter__(self):
        self._initialize()
        self._initialized = True
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._finalize()
        self._initialized = False

    @abstractmethod
    def _initialize(self):
        """Prepare the processor (authenticate, open SDK clients, etc.)."""

    @abstractmethod
    def _finalize(self):
        """Clean up resources."""

    @abstractmethod
    def _fetch_documents(self) -> Iterator[DocumentStream]:
        """Yield documents from the source."""

    def iterate_documents(self) -> Iterator[DocumentStream]:
        if not self._initialized:
            raise RuntimeError(
                "Processor not initialized. Use 'with' to open it first."
            )
        yield from self._fetch_documents()



from typing import Annotated, Literal

from pydantic import AnyHttpUrl, BaseModel, Field

from docling_jobkit.datamodel.google_drive_coords import GoogleDriveCoordinates
from docling_jobkit.datamodel.s3_coords import S3Coordinates


class InBodyTarget(BaseModel):
    kind: Literal["inbody"] = "inbody"


class ZipTarget(BaseModel):
    kind: Literal["zip"] = "zip"


class S3Target(S3Coordinates):
    kind: Literal["s3"] = "s3"


class GoogleDriveTarget(GoogleDriveCoordinates):
    kind: Literal["google_drive"] = "google_drive"


class PutTarget(BaseModel):
    kind: Literal["put"] = "put"
    url: AnyHttpUrl


TaskTarget = Annotated[
    InBodyTarget | ZipTarget | S3Target | GoogleDriveTarget | PutTarget,
    Field(discriminator="kind"),
]
